#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['twitchat']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.24.0', 'irc>=18.0.0']

entry_points = \
{'console_scripts': ['twitchat = twitchat.twitchbot:main',
                     'twitchat-settings = twitchat.settings:main']}

setup(name='twitchat',
      version='1.2.11',
      description="A simple Twitch chat bot that gives you the option to run complicated commands, to an extent that other bots don't allow.",
      author='Fitti',
      author_email='dev@fitti.io',
      url='https://github.com/Fittiboy/python-twitch-bot',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.8',
     )
